@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2023, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -606,8 +606,7 @@ std::string ParseEventType(EventType eventType)
 
 
 CEventQueue::CEventQueue()
-    : m_mutex{},
-      m_fifo(),
+    : m_fifo(),
       m_head{0},
       m_tail{0},
       m_total{0}
@@ -625,15 +624,13 @@ bool CEventQueue::IsEmpty()
     Else, adds the event to the queue and returns \c true. */
 bool CEventQueue::AddEvent(Event&& event)
 {
-    bool result{};
-
-    SDL_LockMutex(*m_mutex);
+    std::lock_guard<std::mutex> lock{m_mutex};
 
     if (m_total >= MAX_EVENT_QUEUE)
     {
         GetLogger()->Warn("Event queue flood!\n");
 
-        result = false;
+        return false;
     }
     else
     {
@@ -644,19 +641,15 @@ bool CEventQueue::AddEvent(Event&& event)
 
         m_total++;
 
-        result = true;
+        return true;
     }
-
-    SDL_UnlockMutex(*m_mutex);
-
-    return result;
 }
 
 Event CEventQueue::GetEvent()
 {
     Event event;
 
-    SDL_LockMutex(*m_mutex);
+    std::lock_guard<std::mutex> lock{m_mutex};
 
     if (m_head == m_tail)
     {
@@ -673,7 +666,5 @@ Event CEventQueue::GetEvent()
 
     }
 
-    SDL_UnlockMutex(*m_mutex);
-
     return event;
 }
