@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2023, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -29,22 +29,3 @@ SystemDialogResult CSystemUtilsOther::SystemDialog(SystemDialogType type, const
     return ConsoleSystemDialog(type, title, message);
 }
 
-void CSystemUtilsOther::GetCurrentTimeStamp(SystemTimeStamp* stamp)
-{
-    stamp->sdlTicks = SDL_GetTicks();
-}
-
-void CSystemUtilsOther::InterpolateTimeStamp(SystemTimeStamp *dst, SystemTimeStamp *a, SystemTimeStamp *b, float i)
-{
-    dst->sdlTicks = a->sdlTicks + static_cast<Uint32>((b->sdlTicks - a->sdlTicks) * i);
-}
-
-long long int CSystemUtilsOther::TimeStampExactDiff(SystemTimeStamp* before, SystemTimeStamp* after)
-{
-    return (after->sdlTicks - before->sdlTicks) * 1000000ll;
-}
-
-void CSystemUtilsOther::Usleep(int usec)
-{
-    SDL_Delay(usec / 1000); // close enough
-}
