@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2023, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -52,7 +52,7 @@ enum Phase
     PHASE_WELCOME2,
     PHASE_WELCOME3,
     PHASE_PLAYER_SELECT,
-    PHASE_APPERANCE,
+    PHASE_APPEARANCE,
     PHASE_MAIN_MENU,
     PHASE_LEVEL_LIST,
     PHASE_MOD_LIST,
@@ -198,7 +198,7 @@ public:
     Phase       GetPhase();
     //@}
 
-    //! Load the scene for apperance customization
+    //! Load the scene for appearance customization
     void        ScenePerso();
 
     void        SetMovieLock(bool lock);
@@ -368,6 +368,9 @@ public:
     //! Enable mode where completing mission closes the game
     void        SetExitAfterMission(bool exit);
 
+    //! Load saved game (used by command line argument)
+    void        LoadSaveFromDirName(const std::string& gameDir);
+
     //! Returns true if player can interact with things manually
     bool        CanPlayerInteract();
 
