@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2023, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -100,7 +100,7 @@ float HaltToFloat(uint16_t value)
     // NaN
     else if ((exponent == 31) && (mantissa != 0))
     {
-        result = std::nanf("");
+        result = nanf("");
     }
     // Normal number
     else
