@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2020, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2023, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -183,7 +183,7 @@ bool CScreenPlayerSelect::EventProcess(const Event &event)
 
         case EVENT_INTERFACE_PERSO:
             NameSelect();
-            m_main->ChangePhase(PHASE_APPERANCE);
+            m_main->ChangePhase(PHASE_APPEARANCE);
             break;
 
         case EVENT_INTERFACE_NDELETE:
